@@ -291,6 +291,13 @@ AnxClip * anx_clip_clone (AnxClip * clip);
  */
 AnxMetaElement * anx_meta_element_clone (AnxMetaElement * meta);
 
+/**
+ * Clone an AnxLinkElement structure
+ * \param link the link tag structure to clone
+ * \returns a new link tag structure
+ */
+AnxLinkElement * anx_link_element_clone (AnxLinkElement * link);
+
 #ifdef __cplusplus
 }
 #endif
