@@ -109,6 +109,7 @@ struct _AnxOggTrack {
   int need_keygranule;
   ogg_int64_t keygranule;
   double keygranule_time;
+  int filter_got_keygranule;
 };
 
 struct _AnxOggData {
@@ -522,8 +523,7 @@ read_packet_headers (OGGZ * oggz, ogg_packet * op, long serialno,
 }
 
 static int
-filter (AnxOggData * aod, AnxOggTrack * aot, long serialno,
-	ogg_int64_t granulepos)
+filter (AnxOggData * aod, AnxOggTrack * aot, long serialno, ogg_packet * op)
 {
   AnxSourceTrack * track = &(aot->source_track);
   double timestamp;
@@ -533,9 +533,8 @@ filter (AnxOggData * aod, AnxOggTrack * aot, long serialno,
 
   /* TRUE if we haven't done the seek yet */
   if (aod->need_seek == NEED_SEEK_PENDING) return 1;
-  
-  timestamp =
-    gp_to_time (aod->oggz, serialno, granulepos);
+
+  timestamp = gp_to_time (aod->oggz, serialno, op->granulepos);
 
   /* TRUE if we're beyond the start time */
   if (timestamp-TOLERANCE >= aod->anx_source->start_time) {
@@ -551,11 +550,37 @@ filter (AnxOggData * aod, AnxOggTrack * aot, long serialno,
   /* FALSE if this track doesn't use granuleshift */
   if (track->granuleshift == 0) return 0;
 
+  /* TRUE if we're beyond the keygranule for this track */
+  if (aot->filter_got_keygranule) return 1;
+
+  /* TRUE if we're after the keygranule time */
+  if (op->granulepos != -1 && timestamp+TOLERANCE >= aot->keygranule_time) {
+    aot->filter_got_keygranule = 1;
+    return 1;
+  }
+
+  /* XXX: TRUE if a theora from a keyframe, even if no granulepos */
+  if (!strcmp (track->content_type, THEORA_CONTENT_TYPE) &&
+      op->bytes > 0 && !(op->packet[0] & 0x40)) {
+#ifdef DEBUG
+    fprintf (aod->df,
+	     "anxogg::FILTER got Theora keyframe\n");
+#endif
+
+    aot->filter_got_keygranule = 1;
+    return 1;
+  }
+
+  /* FALSE otherwise */
+  return 0;
+
+#if 0
   /* FALSE if this track uses keygranules, and we're before its time */
   if (granulepos != -1 && timestamp+TOLERANCE < aot->keygranule_time) return 0;
 
   /* TRUE otherwise */
   return 1;
+#endif
 }
 
 static int
@@ -665,7 +690,7 @@ read_packet_data (OGGZ * oggz, ogg_packet * op, long serialno,
 
   if (!aod->ignore_media && !aod->got_end && !aot->anxv2_ignore_packet &&
       (aod->state != STATE_FILTER ||
-       filter (aod, aot, serialno, op->granulepos))) {
+       filter (aod, aot, serialno, op))) {
 #ifdef DEBUG
     fprintf (aod->df, "anxogg::read_packet_data: no, copy out\n");
 #endif
@@ -799,6 +824,18 @@ read_page_granuleinfo (OGGZ * oggz, const ogg_page * og, long serialno,
     if (aot->need_keygranule && granulepos != -1) {
       iframe = granulepos >> track->granuleshift;
       pframe = granulepos - (iframe << track->granuleshift);
+
+#if 1
+      /* XXX: vaguely similar reasoning to CMML above */
+      start_time = aod->anx_source->start_time;
+      offset = gp_to_time (aod->oggz, serialno, granulepos);
+      if (offset-TOLERANCE <= start_time) {
+	return OGGZ_STOP_OK;
+      }
+      /* End XXX: */
+#endif
+
+
       granulepos = (iframe + pframe);
       aot->keygranule = iframe << track->granuleshift;
       aot->need_keygranule = 0;
@@ -810,8 +847,8 @@ read_page_granuleinfo (OGGZ * oggz, const ogg_page * og, long serialno,
       
 #ifdef DEBUG
       fprintf (aod->df,
-	       "read_page_granuleinfo: ^^^ has keygranule %llx (%f seconds) (granuleshift %d)\n",
-	      aot->keygranule, offset, track->granuleshift);
+	       "read_page_granuleinfo: ^^^ has keygranule %lld (%lld|0) (%f seconds) (granuleshift %d)\n",
+	      aot->keygranule, iframe, offset, track->granuleshift);
 #endif
     }
   }
