@@ -36,8 +36,6 @@
 #include <string.h>
 
 #include <stdlib.h>
-#define anx_malloc malloc
-#define anx_free free 
 
 /*#define DEBUG*/
 
@@ -87,6 +85,14 @@ le_short(unsigned short s)
    return ret;
 }
 
+static char *
+anxsf_strdup (const char * s)
+{
+  char * ret;
+  if (!s) return NULL;
+  ret = anx_malloc (strlen (s) + 1);
+  return strcpy (ret, s);
+}
 
 static void
 anxsf_generate_wav_header (AnxSndfileData * sfd, int rate, int channels,
@@ -177,11 +183,16 @@ anxsf_open (const char * path, const char * id, int ignore_media,
   m->start_time = start_time;
   m->end_time = end_time;
   m->current_time = start_time;
+  m->written_secondaries = 0;
+
+  m->duration = 0.0;
+  m->byte_length = 0;
 
   sfd = anx_malloc (sizeof (AnxSndfileData));
   m->custom_data = sfd;
 
   sf_info = anx_malloc (sizeof (SF_INFO));
+  memset (sf_info, 0, sizeof (SF_INFO));
   sfd->sf_info = sf_info;
 
   sfd->sndfile = sf_open (path, SFM_READ, sf_info);
@@ -195,8 +206,8 @@ anxsf_open (const char * path, const char * id, int ignore_media,
   anxsf_generate_wav_header (sfd, sf_info->samplerate, sf_info->channels,
 			     sf_info->frames);
 
-  track->id = (char *)id; /* XXX: dup? */
-  track->content_type = "audio/x-wav";
+  track->id = anxsf_strdup (id);
+  track->content_type = anxsf_strdup ("audio/x-wav");
 
   track->granule_rate_n = sf_info->samplerate;
   track->granule_rate_d = 1;
@@ -204,6 +215,10 @@ anxsf_open (const char * path, const char * id, int ignore_media,
   track->current_granule = 0;
   track->eos = 0;
 
+  track->basegranule = 0;
+  track->preroll = 0;
+  track->granuleshift = 0;
+
   if (end_time != -1.0) {
     track->end_granule = end_time * sf_info->samplerate;
   } else {
@@ -298,6 +313,7 @@ anxsf_read (AnxSource * media, char * buf, long n, long bound)
     sfd->header_remaining -= header_n;
     n = header_n;
   } else {
+    media->written_secondaries = 1;
 
     frames_n = n / (sfd->sf_info->channels *  sizeof (short));
 
@@ -364,6 +380,15 @@ anxsf_sizeof_next_read (AnxSource * media, long bound)
   }
 }
 
+static AnxSourceTrack *
+anxsf_source_track_free (AnxSourceTrack * track)
+{
+  anx_free (track->id);
+  anx_free (track->content_type);
+  anx_free (track);
+  return NULL;
+}
+
 static int
 anxsf_close (AnxSource * media)
 {
@@ -372,7 +397,9 @@ anxsf_close (AnxSource * media)
   sf_close (sfd->sndfile);
 
   anx_free (sfd->sf_info);
+  anx_free (sfd);
 
+  anx_list_free_with (media->tracks, (AnxFreeFunc)anxsf_source_track_free);
   anx_free (media);
 
   return 0;
