@@ -44,6 +44,9 @@
 
 /*#define DEBUG*/
 
+#define DEFAULT_OGGZ_WRITE_FLAGS (OGGZ_WRITE)
+#define DEFAULT_OGGZ_READ_FLAGS (OGGZ_READ|OGGZ_AUTO)
+
 static int
 anx_flags_check (int flags)
 {
@@ -114,9 +117,9 @@ anx_new (int flags)
     return NULL;
 
   if (flags & ANX_WRITE) {
-    oggz_flags = OGGZ_WRITE;
+    oggz_flags = DEFAULT_OGGZ_WRITE_FLAGS;
   } else {
-    oggz_flags = OGGZ_READ|OGGZ_AUTO;
+    oggz_flags = DEFAULT_OGGZ_READ_FLAGS;
   }
 
   oggz = oggz_new (oggz_flags);
@@ -134,9 +137,9 @@ anx_open (char * filename, int flags)
     return NULL;
 
   if (flags & ANX_WRITE) {
-    oggz_flags = OGGZ_WRITE;
+    oggz_flags = DEFAULT_OGGZ_WRITE_FLAGS;
   } else {
-    oggz_flags = OGGZ_READ|OGGZ_AUTO;
+    oggz_flags = DEFAULT_OGGZ_READ_FLAGS;
   }
 
   oggz = oggz_open (filename, oggz_flags);
@@ -156,9 +159,9 @@ anx_open_stdio (FILE * file, int flags)
     return NULL;
 
   if (flags & ANX_WRITE) {
-    oggz_flags = OGGZ_WRITE;
+    oggz_flags = DEFAULT_OGGZ_WRITE_FLAGS;
   } else {
-    oggz_flags = OGGZ_READ|OGGZ_AUTO;
+    oggz_flags = DEFAULT_OGGZ_READ_FLAGS;
 
     fd = fileno (file);
     if (lseek (fd, 0, SEEK_CUR) != -1) {
