@@ -81,6 +81,48 @@ anx_meta_element_free (AnxMetaElement * meta)
   return NULL;
 }
 
+AnxLinkElement *
+anx_link_element_clone (AnxLinkElement * link)
+{
+  AnxLinkElement * new_link;
+
+  if (link == NULL) return NULL;
+
+  new_link = (AnxLinkElement* ) anx_malloc (sizeof(AnxLinkElement));
+  /* maybe consider memcpy for speedup and introduce reference counting!
+     memcpy (new_link, link, sizeof (*new_link));
+  */
+  new_link->id      = anx_strdup (link->id);
+  new_link->class   = anx_strdup (link->class);
+  new_link->title   = anx_strdup (link->title);
+  new_link->lang    = anx_strdup (link->lang);
+  new_link->dir     = anx_strdup (link->dir);
+  new_link->href    = anx_strdup (link->href);
+  new_link->type    = anx_strdup (link->type);
+  new_link->rel     = anx_strdup (link->rel);
+  new_link->rev     = anx_strdup (link->rev);
+  new_link->media   = anx_strdup (link->media);
+  return new_link;
+}
+
+AnxLinkElement *
+anx_link_element_free (AnxLinkElement * link)
+{
+  if (link == NULL) return NULL;
+  anx_free (link->id);
+  anx_free (link->class);
+  anx_free (link->title);
+  anx_free (link->lang);
+  anx_free (link->dir);
+  anx_free (link->href);
+  anx_free (link->type);
+  anx_free (link->rel);
+  anx_free (link->rev);
+  anx_free (link->media);
+  return NULL;
+}
+
+
 AnxHead *
 anx_head_clone (AnxHead * head)
 {
