@@ -44,6 +44,7 @@ anx_media_open (ANNODEX * anx, char * filename,
 		AnxImportStream import_stream,
 		AnxImportHead import_head,
 		AnxImportClip import_clip,
+		AnxImportCMML import_cmml,
 		void * import_user_data,
 		AnxImportImport import_import,
 		void * import_import_user_data);
@@ -54,6 +55,7 @@ anx_media_openfd (ANNODEX * anx, int fd,
 		  double start, double end, int flags,
 		  AnxImportHead import_head,
 		  AnxImportClip import_clip,
+		  AnxImportCMML import_cmml,
 		  void * import_user_data,
 		  AnxImportImport import_import,
 		  void * import_import_user_data);
@@ -67,12 +69,17 @@ anx_media_query_track (AnxSource * source, int track_index,
 		       char ** id, char ** content_type,
 		       long * nr_header_packets,
 		       anx_int64_t * granule_rate_n,
-		       anx_int64_t * granule_rate_d);
+		       anx_int64_t * granule_rate_d,
+		       anx_int64_t * basegranule,
+		       int * preroll, int * granuleshift);
 
 int anx_media_current_track (AnxSource * source);
 anx_int64_t anx_media_tell (AnxSource * source);
 double anx_media_tell_time (AnxSource * source);
+long anx_media_get_byte_length (AnxSource * media);
+double anx_media_get_duration (AnxSource * media);
 int anx_media_eos (AnxSource * source);
 int anx_media_current_track_eos (AnxSource * media);
+int anx_media_written_secondaries (AnxSource * media);
 
 #endif /* __ANX_IMPORT_PRIVATE_H__ */
