@@ -47,10 +47,16 @@ typedef struct _ANNODEX ANNODEX;
 #include <annodex/annodex.h>
 
 #include "anx_compat.h"
-#include "anx_mapping.h"
-#include "anx_bos.h"
 #include "anx_tree.h"
 
+#include "anx_mapping.h"
+#include "anx_data_packet.h"
+
+#include "cmml_header.h"
+
+#include "fishead.h"
+#include "fisbone.h"
+
 #ifndef MIN
 #define MIN(a,b) ((a)<(b)?(a):(b))
 #endif
@@ -82,6 +88,7 @@ struct _AnxWriterTrack {
   AnxSource * media;
   int media_index;
 
+  int written_fisbone;
   int written_bos;
   int written_eos;
 
@@ -92,17 +99,22 @@ struct _AnxWriter {
   AnxSource * current_media;
   AnxWriterTrack * current_track;
 
-  int need_mapping;
+  int need_fishead;
   int need_bos;
-  int need_anno_bos;
+  int need_fisbone;
+  int need_cmml_fisbone;
+  int need_cmml_bos;
+  int need_cmml_preamble;
   int need_head;
+  int need_mapping_eos;
   int secondary_headers_remaining;
 
   int done_media;
-  int done_anno_eos;
+  int done_cmml_eos;
   int done_eos;
 
-  AnxTrack anno_track;
+  AnxTrack cmml_track;
+  anx_int64_t cmml_prev_granule;
 
   ogg_stream_state mapping_stream;
   ogg_stream_state annotations_stream;
@@ -115,15 +127,14 @@ struct _AnxWriter {
   AnxImportStream import_stream_callback;
   AnxImportHead import_head_callback;
   AnxImportClip import_clip_callback;
+  AnxImportCMML import_cmml_callback;
   void * import_user_data;
 
   AnxImportImport import_import_callback;
   void * import_import_user_data;
 
-  AnxMapping mapping;
-
-  char * anno_buf;
-  long anno_buf_len, anno_buf_n;
+  char * cmml_buf;
+  long cmml_buf_len, cmml_buf_n;
 
   unsigned char * media_buf;
   long media_buf_len, media_buf_n;
@@ -147,7 +158,7 @@ struct _AnxReader {
   void * read_raw_user_data;
 
   AnxList * tracks; /* List of AnxTracks */
-  int got_non_bos;
+  int got_non_init;
   int delivered_all_init_callbacks;
 
   int eos; /* end of stream */
@@ -165,7 +176,8 @@ struct _ANNODEX {
 
   OGGZ * oggz;
 
-  double timebase;
+  double presentation_time;
+  double basetime;
   anx_int64_t current_granule;
   long current_serialno;
 
@@ -178,9 +190,12 @@ struct _ANNODEX {
   int mapping_serialno_valid; /* is mapping_serialno valid? */
   long mapping_serialno; /* serialno of mapping */
 
-  int anno_serialno_valid; /* is anno_serialno valid? */
-  long anno_serialno; /* serialno of annotations stream */ 
+  int cmml_serialno_valid; /* is cmml_serialno valid? */
+  long cmml_serialno; /* serialno of annotations stream */ 
 
+  long total_byte_length;
+  double duration;
+  
   union {
     AnxReader reader;
     AnxWriter writer;
@@ -192,6 +207,7 @@ ANNODEX * anx_writer_init (ANNODEX * annodex);
 ANNODEX * anx_reader_init (ANNODEX * annodex);
 
 int anx_writer_close (ANNODEX * annodex);
+int anx_reader_close (ANNODEX * annodex);
 
 double anx_writer_seek_time (ANNODEX * annodex, double seconds, int whence);
 double anx_reader_seek_time (ANNODEX * annodex, double seconds, int whence);
@@ -201,6 +217,7 @@ int anx_reader_seek_id (ANNODEX * annodex, const char * id);
 
 /* Commands */
 ANNODEX * anx_commands_init (ANNODEX * annodex);
+int anx_commands_delete (ANNODEX * annodex);
 AnxCommand anx_command_seek (ANNODEX * annodex, double time_offset);
 
 /* Errors */
@@ -213,7 +230,7 @@ _anx_ready (ANNODEX * annodex)
   AnxWriter * writer = &annodex->x.writer;
 
   if (annodex->flags == ANX_READ) {
-    if (!reader->got_non_bos) return 0;
+    if (!reader->got_non_init) return 0;
   }
 
   if (annodex->flags == ANX_WRITE) {
