@@ -44,8 +44,8 @@
 #include <time.h>
 
 #include "anx_private.h"
-
-#include "xtag.h"
+#include "anx_element.h"
+#include "anx_snprint.h" /* anx_strdup */
 
 /*#define DEBUG*/
 
@@ -64,6 +64,30 @@ anx_reader_stop_on_head (ANNODEX * annodex, const AnxHead * head,
   return ANX_STOP_OK;
 }
 
+static void
+anx_reader_get_bitrate (ANNODEX * annodex)
+{
+  double start_time, end_time;
+  ogg_int64_t start_unit, end_unit;
+  off_t start_offset, end_offset;
+
+  start_offset = oggz_seek (annodex->oggz, 0, SEEK_CUR);
+  start_unit = oggz_tell_units (annodex->oggz);
+  start_time = ((double)start_unit) / 1000.0;
+
+  end_offset = oggz_seek (annodex->oggz, 0, SEEK_END);
+  end_unit = oggz_seek_units (annodex->oggz, 0, SEEK_END);
+  end_time = ((double)end_unit) / 1000.0;
+
+#ifdef DEBUG
+  printf ("anx_reader_get_bitrate: (%f - %f) [%ld - %ld]\n",
+	  start_time, end_time, start_offset, end_offset);
+#endif
+
+  annodex->duration = end_time - start_time;
+  annodex->total_byte_length = end_offset - start_offset;
+}
+
 static void
 anx_reader_slurp_head (ANNODEX * annodex)
 {
@@ -71,16 +95,21 @@ anx_reader_slurp_head (ANNODEX * annodex)
   AnxReadHead read_head;
 
 #ifdef DEBUG
-  printf ("anx_reader_slurp_head()\n");
+  printf ("anx_reader_slurp_head() IN\n");
 #endif
 
   read_head = reader->read_head;
 
   reader->read_head = anx_reader_stop_on_head;
 
-  while (!anx_ready (annodex) && (anx_read (annodex, 128) > 0));
+  while (!anx_ready (annodex) && (anx_read (annodex, 1024) > 0));
 
   reader->read_head = read_head;
+
+#ifdef DEBUG
+  printf ("anx_reader_slurp_head() OUT: anx_ready ? %s\n",
+	  anx_ready(annodex) ? "TRUE" : "FALSE");
+#endif
 }
 
 int
@@ -230,32 +259,13 @@ anx_reader_call_stream_callback (ANNODEX * annodex)
   int cb_ret = 0;
 
   if (reader->read_stream) {
-    cb_ret = reader->read_stream (annodex, annodex->timebase, NULL /* utc */,
+    cb_ret = reader->read_stream (annodex, annodex->basetime, NULL /* utc */,
 				  reader->read_stream_user_data);
   }
 
   return cb_ret;
 }
 
-static int
-anx_reader_read_mapping (ANNODEX * annodex, unsigned char * data, long n)
-{
-  AnxMapping * mapping = (AnxMapping *)data;
-  anx_int64_t tn, td;
-  /*char * utc = NULL;*/
-
-  if (n < sizeof (AnxMapping)) {
-    /* XXX: header too small! */
-    return -1;
-  }
-
-  tn = _le_64 (mapping->timebase_numerator);
-  td = _le_64 (mapping->timebase_denominator);
-
-  annodex->timebase = (double)tn / (double)td;
-
-  return 0;
-}
 
 static int
 anx_reader_call_track_callback (ANNODEX * annodex, AnxTrack * track)
@@ -264,7 +274,7 @@ anx_reader_call_track_callback (ANNODEX * annodex, AnxTrack * track)
   int cb_ret = 0;
 
   if (reader->read_track) {
-    cb_ret = reader->read_track (annodex, track->serialno,\
+    cb_ret = reader->read_track (annodex, track->serialno,
 				 track->id, track->content_type,
 				 track->granule_rate_n,
 				 track->granule_rate_d, 
@@ -292,11 +302,138 @@ anx_reader_call_track_callbacks (ANNODEX * annodex)
 }
 
 static int
-anx_reader_read_track_bos (ANNODEX * annodex, unsigned char * data, long n,
-			   long serialno)
+anx_reader_read_fishead (ANNODEX * annodex, unsigned char * data, long n)
+{
+  anx_int64_t tn, td;
+  /*char * utc = NULL;*/
+
+  if (n < sizeof (FISHEAD_SIZE)) {
+    /* XXX: header too small! */
+    return -1;
+  }
+
+  tn = _le_64 (*((ogg_int64_t *)&data[12]));
+  td = _le_64 (*((ogg_int64_t *)&data[20]));
+  annodex->presentation_time = (double)tn / (double)td;
+
+  tn = _le_64 (*((ogg_int64_t *)&data[28]));
+  td = _le_64 (*((ogg_int64_t *)&data[36]));
+  annodex->basetime = (double)tn / (double)td;
+
+  return 0;
+}
+static int
+anx_reader_read_fisbone (ANNODEX * annodex, unsigned char * data, long n,
+			 long serialno)
 {
   AnxReader * reader = &annodex->x.reader;
-  AnxBOS * bos = (AnxBOS *)data;
+  AnxList * l;
+  AnxTrack * track;
+  long ref_serialno;
+  char * buf;
+  AnxParams * params;
+  char * content_type = NULL, * id = NULL;
+  int cb_ret = ANX_CONTINUE;
+  
+  if (n < FISBONE_SIZE) {
+    /* XXX: error */
+    return -1;
+  }
+
+  ref_serialno = _le_32 (*((ogg_int32_t *)&data[12]));
+
+  /* Don't doubly insert track records */
+  for (l = reader->tracks; l; l = l->next) {
+    track = (AnxTrack *)l->data;
+    if (track->serialno == ref_serialno) return 0;
+  }
+
+  /* Ensure params is NUL terminated */
+  buf = anx_malloc (n+1-FISBONE_SIZE);
+  memcpy (buf, (char *)data + FISBONE_SIZE, n-FISBONE_SIZE);
+  buf[n-FISBONE_SIZE] = '\0';
+
+  params = anx_params_new_parse (buf, ANX_PARAMS_HEADERS);
+  content_type = anx_strdup (anx_params_get (params, "Content-Type"));
+  id = anx_strdup (anx_params_get (params, "ID"));
+  anx_params_free (params);
+
+  anx_free (buf);
+
+  track = anx_malloc (sizeof (AnxTrack));
+  track->serialno = ref_serialno;
+  track->id = id;
+  track->content_type = content_type;
+  track->granule_rate_n = _le_64 (*((ogg_int64_t *)&data[20]));
+  track->granule_rate_d = _le_64 (*((ogg_int64_t *)&data[28]));
+  track->nr_header_packets = _le_32 (*((ogg_int32_t *)&data[16]));
+  track->basegranule = _le_64 (*((ogg_int64_t *)&data[36]));
+  track->preroll = (int)_le_32 (*((ogg_int32_t *)&data[44]));
+  track->granuleshift = (int)(*((unsigned char *)&data[48]));
+
+  reader->tracks = anx_list_append (reader->tracks, track);
+
+#ifdef DEBUG
+  printf ("Got fisbone for serialno %010ld\n\tgranule_rate %ld/%ld\t%ld headers\t%s\t%s\n",
+	  track->serialno,
+	  (long)track->granule_rate_n,
+	  (long)track->granule_rate_d,
+	  track->nr_header_packets, content_type, id);
+  printf ("\tbasegranule %lld\tpreroll %d\tgranuleshift %d\n",
+	  track->basegranule, track->preroll, track->granuleshift);
+#endif
+
+#if 0
+  if (reader->read_track) {
+    cb_ret = reader->read_track (annodex, track->serialno, id, content_type,
+				 track->granule_rate_n,
+				 track->granule_rate_d, 
+				 track->nr_header_packets,
+				 reader->user_data);
+#ifdef DEBUG
+  } else {
+    printf ("anx_reader_read_fisbone: no read_track callback\n");
+#endif
+  }
+#endif
+
+  if (annodex->cmml_serialno_valid == 0 && content_type &&
+      !strncmp (content_type, "text/x-cmml", 15)) {
+#ifdef DEBUG
+    printf ("Got anno track %010ld\n", track->serialno);
+#endif
+    annodex->cmml_serialno_valid = 1;
+    annodex->cmml_serialno = track->serialno;
+  }
+  
+  return cb_ret;
+}
+
+static int
+anx_reader_read_mapping (ANNODEX * annodex, unsigned char * data, long n)
+{
+  AnxMapping * mapping = (AnxMapping *)data;
+  anx_int64_t tn, td;
+  /*char * utc = NULL;*/
+
+  if (n < sizeof (AnxMapping)) {
+    /* XXX: header too small! */
+    return -1;
+  }
+
+  tn = _le_64 (mapping->timebase_numerator);
+  td = _le_64 (mapping->timebase_denominator);
+
+  annodex->basetime = (double)tn / (double)td;
+
+  return 0;
+}
+static int
+anx_reader_read_track_anxdata (ANNODEX * annodex, unsigned char * data, long n,
+			       long serialno)
+{
+  AnxReader * reader = &annodex->x.reader;
+  AnxData * anxdata = (AnxData *)data;
   AnxList * l;
   AnxTrack * track;
   char * buf;
@@ -304,7 +441,7 @@ anx_reader_read_track_bos (ANNODEX * annodex, unsigned char * data, long n,
   char * content_type = NULL, * id = NULL;
   int cb_ret = ANX_CONTINUE;
   
-  if (n < ANX_BOS_SIZE) {
+  if (n < ANX_DATA_SIZE) {
     /* XXX: error */
     return -1;
   }
@@ -315,146 +452,85 @@ anx_reader_read_track_bos (ANNODEX * annodex, unsigned char * data, long n,
     if (track->serialno == serialno) return 0;
   }
 
-  buf = (char *)data + ANX_BOS_SIZE;
-  params = anx_params_new_parse (buf, ANX_PARAMS_HEADERS);
-  content_type = anx_params_get (params, "Content-Type");
-  id = anx_params_get (params, "ID");
+  /* Ensure params is NUL terminated */
+  buf = anx_malloc (n+1-ANX_DATA_SIZE);
+  memcpy (buf, (char *)data + ANX_DATA_SIZE, n-ANX_DATA_SIZE);
+  buf[n-ANX_DATA_SIZE] = '\0';
 
-#ifdef DEBUG
-  printf ("Got BOS granule_rate %ld/%ld\t%d headers\t%s\t%s\n",
-	  (long)bos->granule_rate_numerator,
-	  (long)bos->granule_rate_denominator,
-	  bos->nr_header_packets, content_type, id);
-#endif
+  params = anx_params_new_parse (buf, ANX_PARAMS_HEADERS);
+  content_type = anx_strdup (anx_params_get (params, "Content-Type"));
+  id = anx_strdup (anx_params_get (params, "ID"));
+  anx_params_free (params);
+
+  anx_free (buf);
 
   track = anx_malloc (sizeof (AnxTrack));
   track->serialno = serialno;
   track->id = id;
   track->content_type = content_type;
-  track->granule_rate_n = _le_64 (bos->granule_rate_numerator);
-  track->granule_rate_d = _le_64 (bos->granule_rate_denominator);
-  track->nr_header_packets = _le_32 (bos->nr_header_packets);
+  track->granule_rate_n = _le_64 (anxdata->granule_rate_numerator);
+  track->granule_rate_d = _le_64 (anxdata->granule_rate_denominator);
+  track->nr_header_packets = _le_32 (anxdata->nr_header_packets);
 
   reader->tracks = anx_list_append (reader->tracks, track);
 
+#ifdef DEBUG
+  printf ("\tAnxData for %010ld: granule_rate %lld/%lld\t%d headers\t%s\t%s\n",
+	  serialno,
+	  track->granule_rate_n, track->granule_rate_d,
+	  track->nr_header_packets, content_type, id);
+#endif
+
 #if 0
   if (reader->read_track) {
-    cb_ret = reader->read_track (annodex, serialno, id, content_type,
+    cb_ret = reader->read_track (annodex, track->serialno, id, content_type,
 				 track->granule_rate_n,
 				 track->granule_rate_d, 
 				 track->nr_header_packets,
 				 reader->user_data);
 #ifdef DEBUG
   } else {
-    printf ("anx_reader_read_sub_bos: no read_track callback\n");
+    printf ("anx_reader_read_sub_anxdata: no read_track callback\n");
 #endif
   }
 #endif
 
-  if (annodex->anno_serialno_valid == 0 && content_type &&
+  if (annodex->cmml_serialno_valid == 0 && content_type &&
       !strncmp (content_type, "text/x-cmml", 15)) {
-    annodex->anno_serialno_valid = 1;
-    annodex->anno_serialno = serialno;
+#ifdef DEBUG
+    printf ("Got anno track %010ld\n", serialno);
+#endif
+    annodex->cmml_serialno_valid = 1;
+    annodex->cmml_serialno = serialno;
   }
   
   return cb_ret;
 }
 
-static AnxHead *
-anx_head_from_xtag (XTag * tag)
+static int
+anx_reader_read_cmml_header (ANNODEX * annodex, unsigned char * data, long n,
+			     long serialno)
 {
-  AnxHead * head;
-  AnxMetaElement * meta;
-  XTag * child;
+  AnxReader * reader = &annodex->x.reader;
+  AnxTrack * track;
 
-  if (tag == NULL) return NULL;
+  annodex->cmml_serialno = serialno;
+  annodex->cmml_serialno_valid = 1;
 
-  head = anx_malloc (sizeof(AnxHead));
+  track = anx_malloc (sizeof (AnxTrack));
+  track->serialno = serialno;
+  track->id = NULL;
+  track->content_type = anx_strdup (CMML_CONTENT_TYPE);
+  track->granule_rate_n = _le_64 (*((ogg_int64_t *)&data[12]));
+  track->granule_rate_d = _le_64 (*((ogg_int64_t *)&data[20]));
+  track->nr_header_packets = 3;
+  track->basegranule = 0;
+  track->preroll = 0;
+  track->granuleshift = (int)(*((unsigned char *)&data[28]));
 
-  head->head_id = xtag_get_attribute (tag, "id");
-  head->lang = xtag_get_attribute (tag, "lang");
-  head->dir = xtag_get_attribute (tag, "dir");
-  head->profile = xtag_get_attribute (tag, "profile");
+  reader->tracks = anx_list_append (reader->tracks, track);
 
-  child = xtag_first_child (tag, "title");
-  head->title = xtag_get_pcdata (child);
-  head->title_id = xtag_get_attribute (child, "id");
-  head->title_lang = xtag_get_attribute (child, "lang");
-  head->title_dir = xtag_get_attribute (child, "dir");
-
-  child = xtag_first_child (tag, "base");
-  head->base_id = xtag_get_attribute (child, "id");
-  head->base_href = xtag_get_attribute (child, "href");
-
-  head->meta = anx_list_new ();
-
-  for (child = xtag_first_child (tag, "meta"); child;
-       child = xtag_next_child (tag, "meta")) {
-    meta = anx_malloc (sizeof (AnxMetaElement));
-    meta->id = xtag_get_attribute (child, "id");
-    meta->lang = xtag_get_attribute (child, "lang");
-    meta->dir = xtag_get_attribute (child, "dir");
-    meta->name = xtag_get_attribute (child, "name");
-    meta->content = xtag_get_attribute (child, "content");
-    meta->scheme = xtag_get_attribute (child, "scheme");
-
-    head->meta = anx_list_append (head->meta, meta);
-  }
-
-  return head;
-}
-
-static AnxClip *
-anx_clip_from_xtag (XTag * tag)
-{
-  AnxClip * clip;
-  AnxMetaElement * meta;
-  XTag * child;
-
-  clip = anx_malloc (sizeof(AnxClip));
-
-  clip->clip_id = xtag_get_attribute (tag, "id");
-  clip->lang = xtag_get_attribute (tag, "lang");
-  clip->dir = xtag_get_attribute (tag, "dir");
-  clip->track = xtag_get_attribute (tag, "track");
-
-  child = xtag_first_child (tag, "a");
-  clip->anchor_id = xtag_get_attribute (child, "id");
-  clip->anchor_lang = xtag_get_attribute (child, "lang");
-  clip->anchor_dir = xtag_get_attribute (child, "dir");
-  clip->anchor_class = xtag_get_attribute (child, "class");
-  clip->anchor_href = xtag_get_attribute (child, "href");
-  clip->anchor_text = xtag_get_pcdata (child);
-
-  child = xtag_first_child (tag, "img");
-  clip->img_id = xtag_get_attribute (child, "id");
-  clip->img_lang = xtag_get_attribute (child, "lang");
-  clip->img_dir = xtag_get_attribute (child, "dir");
-  clip->img_src = xtag_get_attribute (child, "src");
-  clip->img_alt = xtag_get_attribute (child, "alt");
-
-  clip->meta = anx_list_new ();
-
-  for (child = xtag_first_child (tag, "meta"); child;
-       child = xtag_next_child (tag, "meta")) {
-    meta = anx_malloc (sizeof (AnxMetaElement));
-    meta->id = xtag_get_attribute (child, "id");
-    meta->lang = xtag_get_attribute (child, "lang");
-    meta->dir = xtag_get_attribute (child, "dir");
-    meta->name = xtag_get_attribute (child, "name");
-    meta->content = xtag_get_attribute (child, "content");
-    meta->scheme = xtag_get_attribute (child, "scheme");
-
-    clip->meta = anx_list_append (clip->meta, meta);
-  }
-
-  child = xtag_first_child (tag, "desc");
-  clip->desc_id = xtag_get_attribute (child, "id");
-  clip->desc_lang = xtag_get_attribute (child, "lang");
-  clip->desc_dir = xtag_get_attribute (child, "dir");
-  clip->desc_text = xtag_get_pcdata (child);
-
-  return clip;
+  return ANX_CONTINUE;
 }
 
 static int
@@ -481,46 +557,71 @@ anx_read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
   }
 #endif
 
-  if (op->b_o_s) {
-#ifdef DEBUG
-    printf ("\tread_packet: b_o_s\n");
-#endif
-    if (op->bytes >= 8) {
-      if (!strncmp ((char *)op->packet, ANX_MAPPING_IDENTIFIER, 8)) {
-	/* ANNODEX mapping */
-	annodex->mapping_serialno_valid = 1;
-	annodex->mapping_serialno = serialno;
-	anx_reader_read_mapping (annodex, op->packet, op->bytes);
-      } else if (!strncmp ((char *)op->packet, ANX_BOS_IDENTIFIER, 8)) {
-	/* AnxData BOS */
-	cb_ret = anx_reader_read_track_bos (annodex, op->packet, op->bytes,
-						serialno);
-      } else {
-	/* Unknown BOS */
-      }
-    }
+  if (op->b_o_s && op->bytes >= 8 &&
+      !strncmp ((char *)op->packet, FISHEAD_IDENTIFIER, 8)) {
+    /* fishead mapping */
+    annodex->mapping_serialno_valid = 1;
+    annodex->mapping_serialno = serialno;
+    anx_reader_read_fishead (annodex, op->packet, op->bytes);
     return cb_ret;
+  } else if (op->bytes > 8 &&
+	     !strncmp ((char *)op->packet, FISBONE_IDENTIFIER, 8)) {
+    /* fisbone */
+#ifdef DEBUG
+    printf ("anx_read_packet: reading fisbone ...\n");
+#endif
+    cb_ret = anx_reader_read_fisbone (annodex, op->packet, op->bytes,
+				      serialno);
+    return cb_ret;
+  } else if (op->b_o_s && op->bytes >= 8 &&
+      !strncmp ((char *)op->packet, ANX_MAPPING_IDENTIFIER, 8)) {
+    /* ANNODEX mapping */
+    annodex->mapping_serialno_valid = 1;
+    annodex->mapping_serialno = serialno;
+    anx_reader_read_mapping (annodex, op->packet, op->bytes);
+    return cb_ret;
+  } else if (op->bytes > 8 &&
+	     !strncmp ((char *)op->packet, ANX_DATA_IDENTIFIER, 8)) {
+    /* AnxData */
+#ifdef DEBUG
+    printf ("anx_read_packet: reading anxdata ...\n");
+#endif
+    cb_ret = anx_reader_read_track_anxdata (annodex, op->packet, op->bytes,
+					    serialno);
+    return cb_ret;
+  } else if (op->bytes > 6 &&
+	     !strncmp ((char *)op->packet, "CMML", 4)) {
+    /* CMML */
+    cb_ret = anx_reader_read_cmml_header (annodex, op->packet, op->bytes,
+					  serialno);
+    return cb_ret;
+  } else if (op->e_o_s && annodex->mapping_serialno_valid &&
+	     serialno == annodex->mapping_serialno &&
+	     reader->got_non_init == 0) {
+    oggz_set_data_start (oggz, oggz_tell (oggz));
 
-  } else {
-    if (reader->got_non_bos == 0) {
-      oggz_set_data_start (oggz, oggz_tell (oggz));
-
-      /* If not slurping, ie. this read is under user control, ie. the user
-       * has had an opportunity to set track callbacks etc., then,
-       * if the track callbacks etc. haven't been called yet, call them.
-       */
-      if (reader->read_head != anx_reader_stop_on_head) {
-	if (!reader->delivered_all_init_callbacks) {
-	  cb_ret = anx_reader_call_stream_callback (annodex);
-	  if (cb_ret != ANX_CONTINUE) return cb_ret;
-	  cb_ret = anx_reader_call_track_callbacks (annodex);
-	  if (cb_ret != ANX_CONTINUE) return cb_ret;
-	  reader->delivered_all_init_callbacks = 1;
-	}
+#ifdef DEBUG
+    printf ("anx_read_packet: got non init\n");
+#endif
+    
+    /* If not slurping, ie. this read is under user control, ie. the user
+     * has had an opportunity to set track callbacks etc., then,
+     * if the track callbacks etc. haven't been called yet, call them.
+     */
+    if (reader->read_head != anx_reader_stop_on_head) {
+#ifdef DEBUG
+      printf ("anx_read_packet: read_head != stop_on_head\n");
+#endif
+      if (!reader->delivered_all_init_callbacks) {
+	cb_ret = anx_reader_call_stream_callback (annodex);
+	if (cb_ret != ANX_CONTINUE) return cb_ret;
+	cb_ret = anx_reader_call_track_callbacks (annodex);
+	if (cb_ret != ANX_CONTINUE) return cb_ret;
+	reader->delivered_all_init_callbacks = 1;
       }
-
-      reader->got_non_bos = 1;
     }
+
+    reader->got_non_init = 1;
   }
 
   if (annodex->mapping_serialno_valid == 0) {
@@ -530,14 +631,14 @@ anx_read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
     return 0;
   }
 
-  if (annodex->anno_serialno_valid == 0) {
+  if (annodex->cmml_serialno_valid == 0) {
 #ifdef DEBUG
     printf ("\tread_packet: anno serialno invalid -- breaking out\n");
 #endif
     return 0;
   }
 
-  if (serialno == annodex->anno_serialno) {
+  if (serialno == annodex->cmml_serialno) {
       
 #ifdef DEBUG
     printf ("anx_read_packet: delivering clip (%ld) length %ld\n",
@@ -560,6 +661,7 @@ anx_read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 	    clip = anx_clip_from_xtag (tag);
 	    cb_ret =
 	      reader->read_clip (annodex, clip, reader->read_clip_user_data);
+	    anx_clip_free (clip);
 	  }
 	} else if (!strncasecmp ("head", tag_name, 5)) {
 #ifdef DEBUG
@@ -575,6 +677,7 @@ anx_read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 	  }
 
           anx_set_head (annodex, head);
+	  anx_head_free (head);
 	}
       }
       xtag_free (tag);
@@ -606,30 +709,6 @@ anx_reader_units_to_time (ANNODEX * annodex, ogg_int64_t units)
   return ((double)units / 1000.0);
 }
 
-/* An OGGZ metric for seeking on Ogg framing */
-static ogg_int64_t
-anx_reader_metric (OGGZ * oggz, long serialno, ogg_int64_t granulepos,
-		   void * user_data)
-{
-  ANNODEX * annodex = (ANNODEX *)user_data;
-  int err;
-  anx_int64_t granule_rate_n, granule_rate_d;
-  double seconds;
-  ogg_int64_t units;
-
-  err =
-    anx_track_get_granule_rate (annodex, serialno,
-				&granule_rate_n, &granule_rate_d);
-  if (err == -1) return -1L;
-
-  seconds =
-    (double)granulepos * (double)granule_rate_d / (double)granule_rate_n;
-
-  units = anx_reader_time_to_units (annodex, seconds);
-
-  return units;
-}
-
 ANNODEX *
 anx_reader_init (ANNODEX * annodex)
 {
@@ -639,7 +718,7 @@ anx_reader_init (ANNODEX * annodex)
   memset (reader, 0, sizeof (AnxReader));
 
   reader->tracks = NULL;
-  reader->got_non_bos = 0;
+  reader->got_non_init = 0;
   reader->delivered_all_init_callbacks = 0;
 
   reader->eos = 0;
@@ -648,7 +727,6 @@ anx_reader_init (ANNODEX * annodex)
   reader->offset_head = 0;
 
   oggz_set_read_callback (annodex->oggz, -1, anx_read_packet, annodex);
-  oggz_set_metric (annodex->oggz, -1, anx_reader_metric, annodex);
 
 #ifdef DEBUG
   printf ("anx_reader_init: annodex->slurpable? %s\n",
@@ -657,13 +735,24 @@ anx_reader_init (ANNODEX * annodex)
 
   if (annodex->slurpable) {
     anx_reader_slurp_head (annodex);
+    anx_reader_get_bitrate (annodex);
     anx_reader_seek_head (annodex);
-    reader->got_non_bos = 0;
+    reader->got_non_init = 0;
   }
 
   return annodex;
 }
 
+int
+anx_reader_close (ANNODEX * annodex)
+{
+  AnxReader * reader = &annodex->x.reader;
+
+  anx_list_free_with (reader->tracks, (AnxFreeFunc)anx_track_free);
+
+  return 0;
+}
+
 long
 anx_read (ANNODEX * annodex, long n)
 {
@@ -695,7 +784,7 @@ anx_reader_seek_time (ANNODEX * annodex, double seconds, int whence)
     if (seconds == 0.0) {
       err = anx_reader_seek_head (annodex);
       if (err == -1) return -1.0;
-      else return 0.0; /* XXX: timebase? */
+      else return 0.0; /* XXX: basetime? */
     } else {
       whence = ANX_SEEK_SET;
     }
