@@ -42,7 +42,7 @@
 #include "anx_snprint.h"
 
 char *
-anx_strdup (char * s)
+anx_strdup (const char * s)
 {
   char * ret;
   if (!s) return NULL;
