@@ -330,6 +330,35 @@ anx_next_leaf (AnxNode * node)
   return m;
 }
 
+/*
+ * anx_tree_free_with (ctx, tree, free_func)
+ *
+ * Step through tree 'tree', freeing each node using free_func(), and
+ * also free the tree structure itself.
+ */
+AnxTree *
+anx_tree_free_with (void * ctx, AnxTree * tree, AnxFreeFunc free_func)
+{
+  AnxNode * n, * p;
+
+  if (tree == NULL) return NULL;
+
+  p = tree->root;
+
+  while ((n = anx_next_leaf (p))) {
+    if ((p = n->parent)) {
+      if (p->left == n) p->left = NULL;
+      else p->right = NULL;
+    }
+    free_func (n->data);
+    anx_free (n);
+  }
+
+  anx_free (tree);
+
+  return NULL;
+}
+
 AnxTree *
 anx_tree_free (void * ctx, AnxTree * tree)
 {
