@@ -774,21 +774,13 @@ anx_init_head_packet (ANNODEX * annodex)
 }
 
 static anx_int64_t
-anx_writer_cmml_granulepos (ANNODEX * annodex)
+anx_writer_cmml_granulepos_calc (ANNODEX * annodex, double timestamp)
 {
   AnxWriter * writer = &annodex->x.writer;
   AnxTrack * track = &writer->cmml_track;
-  AnxCommand command;
-  AnxCommandData data;
-  double command_time;
   anx_int64_t granule, granulepos;
 
-  command = annodex->pending_command;
-  if (command == NULL) return -1;
-
-  data = anx_command_get_data (annodex, command);
-  command_time = anx_command_get_time_offset (annodex, command);
-  granule = (anx_int64_t)(command_time * track->granule_rate_n /
+  granule = (anx_int64_t)(timestamp * track->granule_rate_n /
 			  track->granule_rate_d);
 
   granulepos =
@@ -796,7 +788,7 @@ anx_writer_cmml_granulepos (ANNODEX * annodex)
     (granule - writer->cmml_prev_granule);
 
 #ifdef DEBUG
-  printf ("anx_writer_cmml_granulepos: prev_granule %llx, granule %llx, granulepos %llx\n",
+  printf ("anx_writer_cmml_granulepos_calc: prev_granule %llx, granule %llx, granulepos %llx\n",
 	  writer->cmml_prev_granule, granule, granulepos);
 #endif
 
@@ -805,6 +797,22 @@ anx_writer_cmml_granulepos (ANNODEX * annodex)
   return granulepos;
 }
 
+static anx_int64_t
+anx_writer_cmml_granulepos (ANNODEX * annodex)
+{
+  AnxCommand command;
+  AnxCommandData data;
+  double command_time;
+
+  command = annodex->pending_command;
+  if (command == NULL) return -1;
+
+  data = anx_command_get_data (annodex, command);
+  command_time = anx_command_get_time_offset (annodex, command);
+
+  return anx_writer_cmml_granulepos_calc (annodex, command_time);
+}
+
 static long
 anx_init_clip_packet (ANNODEX * annodex)
 {
@@ -1039,13 +1047,10 @@ anx_init_eos_packet (ANNODEX * annodex, long serialno, anx_int64_t granulepos)
 static long
 anx_init_clip_eos_packet (ANNODEX * annodex)
 {
-  AnxWriter * writer = &annodex->x.writer;
-  AnxTrack * track = &writer->cmml_track;
   anx_int64_t granulepos;
 
-  granulepos = (anx_int64_t)(anx_tell_time (annodex) *
-			     track->granule_rate_n /
-			     track->granule_rate_d);
+  granulepos =
+    anx_writer_cmml_granulepos_calc (annodex, anx_tell_time (annodex));
 
   return anx_init_eos_packet (annodex, annodex->cmml_serialno, granulepos);
 }
