@@ -198,7 +198,7 @@ xtag_skip_to (XTagParser * parser, int char_class)
       return TRUE;
     }
 #ifdef DEBUG
-    else printf ("xtag: skipping '%c'\n", s[i]);
+    else printf ("xtag: skipping '%c'\n", *s);
 #endif
   }
 
