@@ -107,12 +107,7 @@ fprintf_time_npt (double seconds)
   min = (int) ((seconds - ((double)hrs * 3600.0)) / 60.0);
   sec = seconds - ((double)hrs * 3600.0)- ((double)min * 60.0);
 
-  /* XXX: %02.3f workaround */
-  if (sec < 10.0) {
-    return fprintf (outfile, "%s%02d:%02d:0%2.3f", sign, hrs, min, sec);
-  } else {
-    return fprintf (outfile, "%s%02d:%02d:%02.3f", sign, hrs, min, sec);
-  }
+  return fprintf (outfile, "%s%02d:%02d:%06.3f", sign, hrs, min, sec);
 }
 
 #if 0
@@ -182,9 +177,12 @@ print_tracks (ANNODEX * anx)
 {
   AnxList * tracks, * l;
   AnxTrack * s;
+  double timebase;
 
   fprintf (outfile, "<stream timebase=\"");
-  fprintf_time_npt (anx_get_timebase (anx));
+  timebase = anx_get_timebase (anx);
+  if (timebase == -1) timebase = 0.0;
+  fprintf_time_npt (timebase);
   fprintf (outfile, "\">\n"); /* XXX: and UTC */
 
   tracks = anx_get_track_list (anx);
